/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "helpfunc.h"
#include "rvtypes.h"


#define STARTPOS_INSTANCE "startpos"

#ifndef PI
#define PI 3.14159265358979323846
#endif

float rotvalue(S_FIN_Data *fd)
{
  float x,z, r, rot;
  double dummy;

  x=fd->rot[2][0];
  z=fd->rot[2][2];

  r=sqrt( SQR(x) + SQR(z) );

  if(r < 0.1)
    return -10.0;

  if( z < 0.0 )
    rot= acos(x/r);
  else
    rot= 2*PI - acos(x/r);

  return modf( rot / (2*PI) + 0.25, &dummy);
}



int dodump(const char *fname)
{
  FILE *f;
  rvulong i, cnt;
  S_FIN_Data fd;

  f=fopen(fname,"rb");
  if(f==NULL)
    {
      perror("Cannot open fin file");
      return 1;
    };

  fread(&cnt, sizeof(rvulong), 1, f);

  for(i=0;i<cnt;i++)
    {
      fread(&fd, S_FIN_DATA_SIZE, 1, f);

      if(strcasecmp(fd.fname, STARTPOS_INSTANCE) == 0)
	{
	  printf("-----------------\nStartpos instance:\n");
	  printf("STARTPOS        %g %g %g\n",
		 fd.position.coord[0], fd.position.coord[1], fd.position.coord[2]);
	  if(rotvalue(&fd) < -1.0)
	    printf("(rotation unknown: Instance points up- or downwards!)\n");
	  else
	    printf("STARTROT        %4.2f\n", rotvalue(&fd));
	  printf("-----------------\n");

	}
      else
	printf("%8s.prm at X %g, Y %g, Z %g\n",
	       fd.fname, fd.position.coord[0], fd.position.coord[1], fd.position.coord[2]);
    };

  fclose(f);
  return 0;
}


int main(int argc, char **argv)
{


  if(argc != 2)
    {
      printf("Usage: findump <filename>\n\n");
      printf("Dumps out contents of .fin files.\n\n");
      printf("Looks for usage of the startpos.prm instance, and prints the values\n");
      printf("for a track's start position. (See readme.txt for details)\n");
      return 1;
    };

  if(!filehastype(argv[1],"fin"))
    {
      printf("Error: Argument must be a .fin file's name.\n");
      return 1;
    };

  return dodump(argv[1]);
}
