/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "helpfunc.h"

FileType filetype(const char *fname)
{
  if(filehastype(fname,"prm"))
    return RV_PRM;
  else if(filehastype(fname,"i-p"))
    return IM_PRM;
  else
    return Auto;
}


bool safe_fread(void *ptr, size_t size, size_t nmemb, FILE *stream, const char *fname)
{
  char msg[200];

  if(fread(ptr,size,nmemb,stream) < nmemb)
    {
      if(ferror(stream))
	{
	  sprintf(msg,"Error while read '%s'",fname);
	  perror(msg);
	  return false;
	}
      else
	{
	  printf("File '%s' is truncated or corrupt.\n",fname);
	  return false;
	};
    };
  return true;
}

bool filehastype(const char *fname, const char *suff)
{
  const char *dot;

  dot=strrchr(fname,'.');
  if(dot == NULL)
    return false;
  dot++;
  return strcasecmp(dot,suff)==0;
}




void strcpy_suffix(char *dst, const char *src, const char *newsuff)
{
  char *dot;

  strcpy(dst,src);

  dot=strrchr(dst,'.');
  if(dst == NULL)
    return;
  dot++;
  strcpy(dot,newsuff);
}
