/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "helpfunc.h"
#include "instance.h"

int main(int argc, char **argv)
{
  RVInstance rvi;
  FileType ft;
  rvfloat lo[3], hi[3];
  rvushort i;
  int j;

  if(argc!=2)
    {
      printf("Usage: rvcenter  <filename>\n\n");
      printf("Sets the grip of an instance to a point near its center.\n");
      return 1;
    };

  if((ft=filetype(argv[1]))==Auto)
    {
      printf("Error: File must be an instance file\n");
      return 1;
    };
  
  if(!rvi.load(argv[1],ft))
    return 1;

  if(rvi.header.num_vecs == 0)
    {
      printf("Error: instance is empty.\n");
      return 1;
    };

  lo[0]=lo[1]=lo[2]=1e40;
  hi[0]=hi[1]=hi[2]=-1e40;

  for(i=0;i<rvi.header.num_vecs;i++)
    for(j=0;j<3;j++)
      {
	if(rvi.vert[i].pos.coord[j] < lo[j]) lo[j] = rvi.vert[i].pos.coord[j];
	if(rvi.vert[i].pos.coord[j] > hi[j]) hi[j] = rvi.vert[i].pos.coord[j];
      };

  /* calc new center point, store in lo[]. */
  for(j=0;j<3;j++)
    lo[j] = (lo[j]+hi[j])/2.0;

  if( SQR(lo[0]) + SQR(lo[1]) + SQR(lo[2]) < 5.0 )
    {
      printf("File not modified: Grip already where I'd put it.\n");
      return 1;
    };


  for(i=0;i<rvi.header.num_vecs;i++)
    for(j=0;j<3;j++)
      rvi.vert[i].pos.coord[j] -= lo[j];

  if(!rvi.save(argv[1],ft))
    return 1;

  printf("File '%s' modified.\n",argv[1]);

  return 0;
}
