/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include "helpfunc.h"
#include "instance.h"


bool shade;
float col[3];


float brightness(rvulong c)
{
  float r,g,b;

  r=(float)( (c & 0xff0000) >> 16 ) / 255.0;
  g=(float)( (c & 0xff0000) >> 16 ) / 255.0;
  b=(float)( (c & 0xff0000) >> 16 ) / 255.0;

  return sqrt( SQR(r) + SQR(g) + SQR(b) );
}



rvulong float2ulong(float c)
{
  rvulong res;

  if(c<0) c=0;
  res=(rvulong)(c * 255.0);
  if(res > 255) res=255;
  return res;
}

void setcolor(rvulong &c, float r, float g, float b)
{
  c= (c & 0xff000000)
    | ( float2ulong(r) << 16 )
    | ( float2ulong(g) << 8  )
    | ( float2ulong(b)       );
}


void tmod(S_RV_Poly *poly, rvlong *ncpt,rvlong *ncps, S_RV_Vertex *vlist)
{
  int npoints= (poly->type & 1) ? 4 : 3;
  int i;
  float bri;

  if(poly->texture == -1)
    for(i=0;i<npoints;i++)
      {
	bri=brightness(poly->color[i]);

	if(shade)
	  setcolor(poly->color[i],
		   col[0] * bri, col[1] * bri, col[2]*bri);
	else
	  setcolor(poly->color[i],
		   col[0], col[1], col[2]);
      };
}




int main(int argc, char **argv)
{
  RVInstance rvi;
  FileType ft;
  int i, asint;

  if(argc!=6)
    {
      printf("Usage: rvcolor  <filename> <shade> <red> <green> <blue>\n\n");
      printf("Sets the base color for instances\n\n");
      printf("<shade> is either 'y' or 'n'. When set to 'y', the color will be\n");
      printf("        shaded according to the current color (use bright colors then!)\n");
      printf("        If 'n', the exact color values will be used.\n\n");
      printf("<red>, <green> and <blue> are numbers between 0 and 255. You may also\n");
      printf("        use higher values when encountering problems with shading.\n");
      return 1;
    };

  if((ft=filetype(argv[1]))==Auto)
    {
      printf("Error: File must be an instance file\n");
      return 1;
    };

  if(toupper(argv[2][0]) == 'Y')
    shade=true;
  else if(toupper(argv[2][0]) == 'N')
    shade=false;
  else
    {
      printf("Error: second argument must be 'y' or 'n'.\n");
      return 1;
    };


  for(i=0;i<3;i++)
    {
      if(sscanf(argv[i+3],"%d",&asint) != 1)
	{
	  printf("Error: argument #%d (",i+3);
	  switch(i)
	    {
	    case 0: printf("red"); break;
	    case 1: printf("green"); break;
	    case 2: printf("blue"); break;
	    };
	  printf(" must be a number.\n");
	  return 1;
	};

      if(asint < 0)
	{
	  printf("Error: Color values must be positive!\n");
	  return 1;
	};

      col[i] = ((float)asint) / 255.0;
    };
  
  if(!rvi.load(argv[1],ft))
    return 1;

  rvi.apply_modifier(tmod);

  if(!rvi.save(argv[1],ft))
    return 1;

  printf("File '%s' modified.\n",argv[1]);

  return 0;
}
