/*
 *  Part of rvminis source code,
 *  Copyright (C) 2002 Gabor Varga (bootes@freemail.hu)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "rvtypes.h"
#include "helpfunc.h"
#include "fin.h"
#include "fan.h"
#include "pan.h"


enum fType { FinFile, FanFile, PanFile };

int main(int argc, char **argv)
{
  S_RV_Vector vec;
  fType ft;

  if(argc!=5)
    {
      printf("Usage: rvmove <filename> <dx> <dy> <dz>\n\n");
      printf("Translates (moves) the contents of Instance/AI Node/Pos Node files.\n");
      printf("<dx>,<dy>,<dz> are translation coordinates (floating point numbers).\n");
      return 1;
    };

  if(filehastype(argv[1], "fin"))
    ft=FinFile;
  else if(filehastype(argv[1], "fan"))
    ft=FanFile;
  else if(filehastype(argv[1], "pan"))
    ft=PanFile;
  else
    {
      printf("Error: File must be a FIN / FAN / PAN file\n");
      return 1;
    }

  if(
      sscanf(argv[2],"%f",&(vec.coord[0]))!=1 ||
      sscanf(argv[3],"%f",&(vec.coord[1]))!=1 ||
      sscanf(argv[4],"%f",&(vec.coord[2]))!=1
    )
    {
      printf("Error: invalid translation vector.\n");
      return 1;
    };

  // Change coordinate system from (X;Y;Z) to (X;Z;-Y).
  rvfloat tmp;
  tmp=vec.coord[1];
  vec.coord[1]=-vec.coord[2];
  vec.coord[2]=tmp;

  switch(ft)
    {
      case FinFile:
        {
          RVFin fin;
          if(!fin.load(argv[1]))
            return 1;
          fin.translate(vec);
          if(!fin.save(argv[1]))
            return 1;
          break;
        }

      case FanFile:
        {
          RVFan fan;
          if(!fan.load(argv[1]))
            return 1;
          fan.translate(vec);
          if(!fan.save(argv[1]))
            return 1;
          break;
        }

      case PanFile:
        {
          RVPan pan;
          if(!pan.load(argv[1]))
            return 1;
          pan.translate(vec);
          if(!pan.save(argv[1]))
            return 1;
          break;
        }

      default:
        {
          printf("Internal error\n");
          return 1;
        }
    }

  printf("File '%s' modified.\n",argv[1]);

  return 0;
}
