/*
 *  Part of rvminis source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "helpfunc.h"
#include "instance.h"



int texrefs[26];
int solidcols=0;
int trans=0;
int envdis=0;
int doubles=0;
int surfs[40];
int nocolls=0;
int invisible=0;

void tmod(S_RV_Poly *p, rvlong *ncpt,rvlong *ncps, S_RV_Vertex *vlist)
{
  if((p->texture>=0) && (p->texture<26))
    texrefs[p->texture]++;
  else if(p->texture == -1)
    solidcols++;
  else if(p->texture == -2)
    invisible++;
  
  if((p->texture != -2) && (p->type & 4))
    trans++;

  if((p->texture != -2) && (p->type & 0x400))
    envdis++;

  if((p->texture != -2) && (p->type & 2))
    doubles++;

  if((*ncps>=0)&&(*ncps<40))
    surfs[*ncps]++;
  else if(*ncps == -2)
    nocolls++;
}










int main(int argc, char **argv)
{
  RVInstance rvi;
  int i;
  bool first;

  if(argc!=2)
    {
      printf("Usage: rvstats <filename>\n\n");
      printf("Prints some infos about instances.\n");
      return 1;
    };

  if(!rvi.load(argv[1],Auto))
    return 1;

  for(i=0;i<26;i++)
    texrefs[i]=0;

  for(i=0;i<40;i++)
    surfs[i]=0;

  rvi.apply_modifier(tmod);


  printf("Number of polygons:         %d\n",rvi.header.num_polys);
  printf("Number of vertices:         %d\n",rvi.header.num_vecs);

  printf("Texture references:         ");
  first=true;
  for(i=0;i<26;i++)
    if(texrefs[i]>0)
      {
	if(!first)
	  printf(", ");
	printf("%dx%c", texrefs[i], 'A'+i);
	first=false;
      };
  printf("\n");
  printf("Solid colored polygons:     %d\n",solidcols);
  printf("Transparent polygons:       %d\n",trans);
  printf("Env-mapping disabled polys: %d\n",envdis);
  printf("Double-sided polygons:      %d\n",doubles);
  printf("Invisible polygons:         ");
  if(filehastype(argv[1],"prm"))
    printf("N/A\n");
  else
    printf("%d\n",invisible);
  printf("Surface types:              ");
  if(filehastype(argv[1],"prm"))
    printf("N/A");
  else
    {
      first=true;
      for(i=0;i<40;i++)
	if(surfs[i]>0)
	  {
	    if(!first)
	      printf(", ");
	    printf("#%d",i);
	    first=false;
	  };
    };
  printf("\n");
  printf("Non-solid polygons:         ");
  if(filehastype(argv[1],"prm"))
    printf("N/A\n");
  else
    printf("%d\n",nocolls);

  return 0;
}
